@@ -56,13 +56,13 @@ class Backend(Database):
     @staticmethod
     def _update_lookups(db):
         for severity, code in alarm_model.Severity.items():
-            db.codes.update(
+            db.codes.update_one(
                 {'severity': severity},
                 {'$set': {'severity': severity, 'code': code}},
                 upsert=True
             )
         for status, state in alarm_model.Status.items():
-            db.states.update(
+            db.states.update_one(
                 {'status': status},
                 {'$set': {'status': status, 'state': state}},
                 upsert=True
@@ -226,7 +226,8 @@ class Backend(Database):
             update['$push'] = {
                 'history': {
                     '$each': [history.serialize],
-                    '$slice': -abs(current_app.config['HISTORY_LIMIT'])
+                    '$slice': current_app.config['HISTORY_LIMIT'],
+                    '$position': 0
                 }
             }
 
@@ -279,7 +280,8 @@ class Backend(Database):
             '$push': {
                 'history': {
                     '$each': [h.serialize for h in history],
-                    '$slice': -abs(current_app.config['HISTORY_LIMIT'])
+                    '$slice': current_app.config['HISTORY_LIMIT'],
+                    '$position': 0
                 }
             }
         }
@@ -347,7 +349,8 @@ class Backend(Database):
             '$push': {
                 'history': {
                     '$each': [h.serialize for h in history],
-                    '$slice': -abs(current_app.config['HISTORY_LIMIT'])
+                    '$slice': current_app.config['HISTORY_LIMIT'],
+                    '$position': 0
                 }
             }
         }
@@ -382,14 +385,14 @@ class Backend(Database):
             '$push': {
                 'history': {
                     '$each': [history.serialize],
-                    '$slice': -abs(current_app.config['HISTORY_LIMIT'])
+                    '$slice': current_app.config['HISTORY_LIMIT'],
+                    '$position': 0
                 }
             }
         }
         return self.get_db().alerts.find_one_and_update(
             query,
             update=update,
-            projection={'history': 0},
             return_document=ReturnDocument.AFTER
         )
 
@@ -408,10 +411,11 @@ class Backend(Database):
         response = self.get_db().alerts.update_one({'_id': {'$regex': '^' + id}}, {'$pullAll': {'tags': tags}})
         return response.matched_count > 0
 
+    def update_tags(self, id, tags):
+        response = self.get_db().alerts.update_one({'_id': {'$regex': '^' + id}}, update={'$set': {'tags': tags}})
+        return response.matched_count > 0
+
     def update_attributes(self, id, old_attrs, new_attrs):
-        """
-        Set all attributes and unset attributes by using a value of 'null'.
-        """
         update = dict()
         set_value = {'attributes.' + k: v for k, v in new_attrs.items() if v is not None}
         if set_value:
@@ -421,8 +425,12 @@ class Backend(Database):
             update['$unset'] = unset_value
 
         if update:
-            response = self.get_db().alerts.update_one({'_id': {'$regex': '^' + id}}, update=update)
-            return response.matched_count > 0
+            return self.get_db().alerts.find_one_and_update(
+                {'_id': {'$regex': '^' + id}},
+                update=update,
+                return_document=ReturnDocument.AFTER
+            )['attributes']
+        return {}
 
     def delete_alert(self, id):
         response = self.get_db().alerts.delete_one({'_id': {'$regex': '^' + id}})
@@ -471,14 +479,14 @@ class Backend(Database):
             '$push': {
                 'history': {
                     '$each': [history.serialize],
-                    '$slice': -abs(current_app.config['HISTORY_LIMIT'])
+                    '$slice': current_app.config['HISTORY_LIMIT'],
+                    '$position': 0
                 }
             }
         }
         return self.get_db().alerts.find_one_and_update(
             query,
             update=update,
-            projection={'history': 0},
             return_document=ReturnDocument.AFTER
         )
 
@@ -566,6 +574,7 @@ class Backend(Database):
                     'origin': response['origin'],
                     'updateTime': response['history']['updateTime'],
                     'user': response['history'].get('user'),
+                    'timeout': response['history'].get('timeout'),
                     'type': response['history'].get('type', 'unknown'),
                     'customer': response.get('customer')
                 }
@@ -585,6 +594,7 @@ class Backend(Database):
             'attributes': 1,
             'origin': 1,
             'user': 1,
+            'timeout': 1,
             'type': 1,
             'history': 1
         }
@@ -619,6 +629,7 @@ class Backend(Database):
                     'origin': response['origin'],
                     'updateTime': response['history']['updateTime'],
                     'user': response.get('user'),
+                    'timeout': response.get('timeout'),
                     'type': response['history'].get('type', 'unknown'),
                     'customer': response.get('customer', None)
                 }
@@ -1488,14 +1499,14 @@ class Backend(Database):
         for match in matches:
             if match in current_app.config['ADMIN_ROLES']:
                 return ADMIN_SCOPES
-            if match == 'user':
+            if match in current_app.config['USER_ROLES']:
                 scopes.extend(current_app.config['USER_DEFAULT_SCOPES'])
-            if match == 'guest':
+            if match in current_app.config['GUEST_ROLES']:
                 scopes.extend(current_app.config['GUEST_DEFAULT_SCOPES'])
             response = self.get_db().perms.find_one({'match': match}, projection={'scopes': 1, '_id': 0})
             if response:
                 scopes.extend(response['scopes'])
-        return sorted(set(scopes)) or current_app.config['USER_DEFAULT_SCOPES']
+        return sorted(set(scopes))
 
     # CUSTOMERS
 
@@ -1678,15 +1689,18 @@ class Backend(Database):
 
         # get list of alerts to be newly expired
         pipeline = [
+            {'$match': {'status': {'$nin': ['expired']}}},
             {'$addFields': {
-                'expireTime': {'$add': ['$lastReceiveTime', {'$multiply': ['$timeout', 1000]}]}}
-             },
-            {'$match': {'status': {'$nin': ['expired', 'ack', 'shelved']}, 'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {
-                '$ne': 0}}}
+                'computedTimeout': {'$multiply': [{'$ifNull': ['$timeout', current_app.config['ALERT_TIMEOUT']]}, 1000]}
+            }},
+            {'$addFields': {
+                'isExpired': {'$lt': [{'$add': ['$lastReceiveTime', '$computedTimeout']}, datetime.utcnow()]}
+            }},
+            {'$match': {'isExpired': True, 'computedTimeout': {'$ne': 0}}}
         ]
         return self.get_db().alerts.aggregate(pipeline)
 
-    def get_timeout(self):
+    def get_unshelve(self):
         # get list of alerts to be unshelved
         pipeline = [
             {'$match': {'status': 'shelved'}},
@@ -1696,13 +1710,47 @@ class Backend(Database):
                 'history.status': 'shelved'
             }},
             {'$sort': {'history.updateTime': -1}},
-            {'$addFields': {
-                'expireTime': {'$add': ['$updateTime', {'$multiply': ['$timeout', 1000]}]}
+            {'$group': {
+                '_id': '$_id',
+                'resource': {'$first': '$resource'},
+                'event': {'$first': '$event'},
+                'environment': {'$first': '$environment'},
+                'severity': {'$first': '$severity'},
+                'correlate': {'$first': '$correlate'},
+                'status': {'$first': '$status'},
+                'service': {'$first': '$service'},
+                'group': {'$first': '$group'},
+                'value': {'$first': '$value'},
+                'text': {'$first': '$text'},
+                'tags': {'$first': '$tags'},
+                'attributes': {'$first': '$attributes'},
+                'origin': {'$first': '$origin'},
+                'type': {'$first': '$type'},
+                'createTime': {'$first': '$createTime'},
+                'timeout': {'$first': '$timeout'},
+                'rawData': {'$first': '$rawData'},
+                'customer': {'$first': '$customer'},
+                'duplicateCount': {'$first': '$duplicateCount'},
+                'repeat': {'$first': '$repeat'},
+                'previousSeverity': {'$first': '$previousSeverity'},
+                'trendIndication': {'$first': '$trendIndication'},
+                'receiveTime': {'$first': '$receiveTime'},
+                'lastReceiveId': {'$first': '$lastReceiveId'},
+                'lastReceiveTime': {'$first': '$lastReceiveTime'},
+                'updateTime': {'$first': '$updateTime'},
+                'history': {'$first': '$history'},
             }},
-            {'$match': {'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {'$ne': 0}}}
+            {'$addFields': {
+                'computedTimeout': {'$multiply': [{'$ifNull': ['$history.timeout', current_app.config['SHELVE_TIMEOUT']]}, 1000]}
+            }},
+            {'$addFields': {
+                'isExpired': {'$lt': [{'$add': ['$updateTime', '$computedTimeout']}, datetime.utcnow()]}
+            }},
+            {'$match': {'isExpired': True, 'computedTimeout': {'$ne': 0}}}
         ]
-        unshelve = self.get_db().alerts.aggregate(pipeline)
+        return self.get_db().alerts.aggregate(pipeline)
 
+    def get_unack(self):
         # get list of alerts to be unack'ed
         pipeline = [
             {'$match': {'status': 'ack'}},
@@ -1712,11 +1760,42 @@ class Backend(Database):
                 'history.status': 'ack'
             }},
             {'$sort': {'history.updateTime': -1}},
-            {'$addFields': {
-                'expireTime': {'$add': ['$updateTime', {'$multiply': ['$timeout', 1000]}]}
+            {'$group': {
+                '_id': '$_id',
+                'resource': {'$first': '$resource'},
+                'event': {'$first': '$event'},
+                'environment': {'$first': '$environment'},
+                'severity': {'$first': '$severity'},
+                'correlate': {'$first': '$correlate'},
+                'status': {'$first': '$status'},
+                'service': {'$first': '$service'},
+                'group': {'$first': '$group'},
+                'value': {'$first': '$value'},
+                'text': {'$first': '$text'},
+                'tags': {'$first': '$tags'},
+                'attributes': {'$first': '$attributes'},
+                'origin': {'$first': '$origin'},
+                'type': {'$first': '$type'},
+                'createTime': {'$first': '$createTime'},
+                'timeout': {'$first': '$timeout'},
+                'rawData': {'$first': '$rawData'},
+                'customer': {'$first': '$customer'},
+                'duplicateCount': {'$first': '$duplicateCount'},
+                'repeat': {'$first': '$repeat'},
+                'previousSeverity': {'$first': '$previousSeverity'},
+                'trendIndication': {'$first': '$trendIndication'},
+                'receiveTime': {'$first': '$receiveTime'},
+                'lastReceiveId': {'$first': '$lastReceiveId'},
+                'lastReceiveTime': {'$first': '$lastReceiveTime'},
+                'updateTime': {'$first': '$updateTime'},
+                'history': {'$first': '$history'},
             }},
-            {'$match': {'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {'$ne': 0}}}
+            {'$addFields': {
+                'computedTimeout': {'$multiply': [{'$ifNull': ['$history.timeout', current_app.config['ACK_TIMEOUT']]}, 1000]}
+            }},
+            {'$addFields': {
+                'isExpired': {'$lt': [{'$add': ['$updateTime', '$computedTimeout']}, datetime.utcnow()]}
+            }},
+            {'$match': {'isExpired': True, 'computedTimeout': {'$ne': 0}}}
         ]
-        unack = self.get_db().alerts.aggregate(pipeline)
-
-        return list(unshelve) + list(unack)
+        return self.get_db().alerts.aggregate(pipeline)
