@@ -129,6 +129,9 @@ class Database(Base):
     def untag_alert(self, id, tags):
         raise NotImplementedError
 
+    def update_tags(self, id, tags):
+        raise NotImplementedError
+
     def update_attributes(self, id, old_attrs, new_attrs):
         raise NotImplementedError
 
@@ -406,7 +409,10 @@ class Database(Base):
     def get_expired(self, expired_threshold, info_threshold):
         raise NotImplementedError
 
-    def get_timeout(self):
+    def get_unshelve(self):
+        raise NotImplementedError
+
+    def get_unack(self):
         raise NotImplementedError
 
 
