@@ -5,7 +5,8 @@ from flask import current_app, g
 
 from alerta.app import plugins
 from alerta.exceptions import (ApiError, BlackoutPeriod, ForwardingLoop,
-                               HeartbeatReceived, RateLimit, RejectException)
+                               HeartbeatReceived, InvalidAction, RateLimit,
+                               RejectException)
 from alerta.models.alert import Alert
 from alerta.models.enums import Scope
 
@@ -63,6 +64,8 @@ def process_alert(alert: Alert) -> Alert:
     except Exception as e:
         raise ApiError(str(e))
 
+    wanted_plugins, wanted_config = plugins.routing(alert)
+
     updated = None
     for plugin in wanted_plugins:
         if skip_plugins:
@@ -80,13 +83,14 @@ def process_alert(alert: Alert) -> Alert:
             alert = updated
 
     if updated:
-        alert.tag(alert.tags)
-        alert.update_attributes(alert.attributes)
+        alert.update_tags(alert.tags)
+        alert.attributes = alert.update_attributes(alert.attributes)
 
     return alert
 
 
-def process_action(alert: Alert, action: str, text: str, timeout: int) -> Tuple[Alert, str, str, int]:
+def process_action(alert: Alert, action: str, text: str, timeout: int = None) -> Tuple[Alert, str, str, Optional[int]]:
+
     wanted_plugins, wanted_config = plugins.routing(alert)
 
     updated = None
@@ -97,7 +101,7 @@ def process_action(alert: Alert, action: str, text: str, timeout: int) -> Tuple[
             updated = plugin.take_action(alert, action, text, timeout=timeout, config=wanted_config)
         except NotImplementedError:
             pass  # plugin does not support action() method
-        except (RejectException, ForwardingLoop):
+        except (RejectException, ForwardingLoop, InvalidAction):
             raise
         except Exception as e:
             if current_app.config['PLUGINS_RAISE_ON_ERROR']:
@@ -113,13 +117,43 @@ def process_action(alert: Alert, action: str, text: str, timeout: int) -> Tuple[
             elif len(updated) == 3:
                 alert, action, text = updated
 
-    # remove keys from attributes with None values
-    new_attrs = {k: v for k, v in alert.attributes.items() if v is not None}
-    alert.attributes = new_attrs
+    if updated:
+        alert.update_tags(alert.tags)
+        alert.attributes = alert.update_attributes(alert.attributes)
 
     return alert, action, text, timeout
 
 
+def process_note(alert: Alert, text: str) -> Tuple[Alert, str]:
+
+    wanted_plugins, wanted_config = plugins.routing(alert)
+
+    updated = None
+    for plugin in wanted_plugins:
+        try:
+            updated = plugin.take_note(alert, text, config=wanted_config)
+        except NotImplementedError:
+            pass  # plugin does not support take_note() method
+        except (RejectException, ForwardingLoop):
+            raise
+        except Exception as e:
+            if current_app.config['PLUGINS_RAISE_ON_ERROR']:
+                raise ApiError("Error while running note plugin '{}': {}".format(plugin.name, str(e)))
+            else:
+                logging.error("Error while running note plugin '{}': {}".format(plugin.name, str(e)))
+
+        if isinstance(updated, Alert):
+            updated = updated, text
+        if isinstance(updated, tuple) and len(updated) == 2:
+            alert, text = updated
+
+    if updated:
+        alert.update_tags(alert.tags)
+        alert.update_attributes(alert.attributes)
+
+    return alert, text
+
+
 def process_status(alert: Alert, status: str, text: str) -> Tuple[Alert, str, str]:
 
     wanted_plugins, wanted_config = plugins.routing(alert)
@@ -145,9 +179,9 @@ def process_status(alert: Alert, status: str, text: str) -> Tuple[Alert, str, st
             except Exception:
                 alert = updated
 
-    # remove keys from attributes with None values
-    new_attrs = {k: v for k, v in alert.attributes.items() if v is not None}
-    alert.attributes = new_attrs
+    if updated:
+        alert.update_tags(alert.tags)
+        alert.attributes = alert.update_attributes(alert.attributes)
 
     return alert, status, text
 
