@@ -13,7 +13,7 @@ from alerta.models.enums import Scope
 from alerta.models.metrics import Timer, timer
 from alerta.models.switch import Switch
 from alerta.utils.api import (assign_customer, process_action, process_alert,
-                              process_delete, process_status)
+                              process_delete, process_note, process_status)
 from alerta.utils.audit import write_audit_trail
 from alerta.utils.paging import Page
 from alerta.utils.response import absolute_url, jsonp
@@ -259,7 +259,7 @@ def update_attributes(alert_id):
 # delete
 @api.route('/alert/<alert_id>', methods=['OPTIONS', 'DELETE'])
 @cross_origin()
-@permission(Scope.write_alerts)
+@permission(Scope.delete_alerts)
 @timer(delete_timer)
 @jsonp
 def delete_alert(alert_id):
@@ -296,6 +296,8 @@ def delete_alert(alert_id):
 def search_alerts():
     query_time = datetime.utcnow()
     query = qb.from_params(request.args, customers=g.customers, query_time=query_time)
+    show_raw_data = request.args.get('show-raw-data', default=False, type=lambda x: x.lower() in ['true', 't', '1', 'yes', 'y', 'on'])
+    show_history = request.args.get('show-history', default=False, type=lambda x: x.lower() in ['true', 't', '1', 'yes', 'y', 'on'])
     severity_count = Alert.get_counts_by_severity(query)
     status_count = Alert.get_counts_by_status(query)
 
@@ -304,6 +306,12 @@ def search_alerts():
 
     alerts = Alert.find_all(query, paging.page, paging.page_size)
 
+    for alert in alerts:
+        if not show_raw_data:
+            alert.raw_data = None
+        if not show_history:
+            alert.history = []
+
     if alerts:
         return jsonify(
             status='ok',
@@ -579,6 +587,7 @@ def add_note(alert_id):
     if not alert:
         raise ApiError('not found', 404)
 
+    alert, note_text = process_note(alert, note_text)
     note = alert.add_note(note_text)
 
     write_audit_trail.send(current_app._get_current_object(), event='alert-note-added', message='', user=g.login,
@@ -646,6 +655,7 @@ def update_note(alert_id, note_id):
                            customers=g.customers, scopes=g.scopes, resource_id=note.id, type='note',
                            request=request)
 
+    _, update['text'] = process_note(alert, update.get('text'))
     updated = note.update(**update)
     if updated:
         return jsonify(status='ok', note=updated.serialize)
